############################################################################
# Script:      SMA200.R
# Objective:   SMA 200 strategy
# Author:      Mario Pisa
# Date:        2019/03/12
# 
# References:  
# TO-DO:       
# install.packages("devtools")
# require(devtools)
# install_github("braverock/blotter") # dependency
# install_github("braverock/quantstrat")
# install.packages("FinancialInstrument")
# install.packages("PerformanceAnalytics")
# install.packages("foreach")
############################################################################
library(quantstrat)

######################
# FUNCTIONS 
######################
"sigAND" <- function(label, data = mktdata, columns,  cross = FALSE) {
  ret_sig = NULL
  colNums <- rep(0, length(columns))
  for(i in 1:length(columns)) {
    colNums[i] <- match.names(columns[i], colnames(data))
  }
  ret_sig <- data[, colNums[1]]
  for(i in 2:length(colNums)) {
    ret_sig <- ret_sig & data[, colNums[i]]
  }
  ret_sig <- ret_sig * 1
  if (isTRUE(cross)) 
    ret_sig <- diff(ret_sig) == 1
  colnames(ret_sig) <- label
  return(ret_sig)
}

initDate = "2006-12-31" 
from = "2007-01-01" 
to = "2019-12-31" 
options(width = 90) 

######################################
# SETUP WORKSPACE AND DOWNLOAD DATA
######################################
# Do some house cleaning
rm(list = ls(.blotter), envir = .blotter)
# Set the currency and the timezone 
currency('EUR')
Sys.setenv(TZ = "UTC")

#IBEX <- getSymbols("^IBEX", src = "yahoo", auto.assign = FALSE)
IBEX <- readRDS("IBEX.rds")
IBEX <- na.omit(IBEX)

symbols <- c("IBEX")
# Define the instrument type
stock(symbols, currency = "EUR", multiplier = 10)

######################
# STRATEGY PARAMETERS 
######################
# Trade sizing and initial equity settings 
tradeSize <- 10000
initEq <- tradeSize * length(symbols)

# indicator parameters
.nSlow <- 200

# signal parameters
.crossAND <- FALSE

# rule parameters
.orderqty <- 1
.enabledStopLoss <- FALSE
.stopLevel <- 0.05

###################
# INIT STRATEGY
###################
strategy.st <- "SMA200" 
portfolio.st <- "SMA200" 
account.st <- "SMA200" 
rm.strat(portfolio.st) 
rm.strat(strategy.st)

initPortf(portfolio.st, symbols = symbols,
          initDate = initDate, currency = 'EUR')

initAcct(account.st, portfolios = portfolio.st,
         initDate = initDate, currency = 'EUR', initEq = initEq)

initOrders(portfolio.st, initDate = initDate)

strategy(strategy.st, store = TRUE)

addPosLimit(
  portfolio = portfolio.st,
  symbol = symbols,
  timestamp = from,
  maxpos = 1,
  minpos = -1
)

###################
# INDICATORS
###################
add.indicator(
  strategy.st,
  name = "EMA",
  arguments = list(
    x = quote(Cl(mktdata)[, 1]),
    n = .nSlow
  ),
  label = "nSlow"
)

# TEST INDICATORS
###################
# test <- applyIndicators(strategy.st, mktdata = get(symbols[1]))
# tail(test)

###############
# SIGNALS
###############
# sigComparison, sigThreshold, sigAND, and sigCrossover.
add.signal(
  strategy.st,
  name = "sigComparison",
  arguments = list(
    columns = c("Close", "nSlow"),
    relationship = "gt"
  ),
  label = "Buy_signal"
)
add.signal(
  strategy.st,
  name = "sigComparison",
  arguments = list(
    columns = c("Close", "nSlow"),
    relationship = "lt"
  ),
  label = "Sell_signal"
)

# TEST Signals
##################
# x <- applyIndicators(strategy.st, mktdata = get(symbols[1]))
# test <- applySignals(strategy.st, mktdata = x)
# tail(test)

###############
# RULES
###############
# Short rules
add.rule(strategy.st,
         name = "ruleSignal",
         arguments = list(sigcol = "Sell_signal", #"shortDaily", #  
                          sigval = TRUE,
                          ordertype = "market",
                          orderside = "short",
                          replace = F,
                          prefer = "Open",
                          osFUN = "osMaxPos",
                          orderqty = -.orderqty,
                          tradeSize = tradeSize),
         type = "enter",
         path.dep = TRUE,
         label = "EnterSHORT")

# add.rule(strategy.st,
#          name = "ruleSignal",
#          arguments = list(sigcol = "exitShort",
#                           sigval = TRUE,
#                           orderqty = "all",
#                           ordertype = "market",
#                           orderside = "short",
#                           replace = F,
#                           prefer = "Open"),
#          type = "exit",
#          path.dep = TRUE,
#          label = "ExitSHORT")

# Long rules
add.rule(strategy.st,
         name = "ruleSignal",
         arguments = list(sigcol = "Buy_signal", # "longDaily", # 
                          sigval = TRUE,
                          ordertype = "market",
                          orderside = "long",
                          replace = F,
                          prefer = "Open",
                          osFUN = "osMaxPos",
                          orderqty = .orderqty,
                          tradeSize = tradeSize),
         type = "enter",
         path.dep = TRUE,
         label = "EnterLONG")

# add.rule(strategy.st,
#          name = "ruleSignal",
#          arguments = list(sigcol = "exitLong", # "sellOrShort",
#                           sigval = TRUE,
#                           orderqty = "all",
#                           ordertype = "market",
#                           orderside = "long",
#                           replace = F,
#                           prefer = "Open"),
#          type = "exit",
#          path.dep = TRUE,
#          label = "ExitLONG")

# Rule for Stop-loss
add.rule(
  strategy.st,
  name = 'ruleSignal',
  arguments = list(
    sigcol = 'Sell_signal' ,
    sigval = TRUE,
    replace = FALSE,
    orderside = 'short',
    ordertype = 'stoptrailing', # 'stoplimit', # 'stoptrailing'
    tmult = TRUE,
    threshold = quote(.stopLevel),
    # TxnFees = .txnfees,
    orderqty = 'all',
    orderset = 'ocoshort'
  ),
  type = 'chain',
  parent = 'EnterSHORT',
  label = 'StopLossSHORT',
  enabled = .enabledStopLoss
)

add.rule(
  strategy.st,
  name = 'ruleSignal',
  arguments = list(
    sigcol = 'Buy_signal' ,
    sigval = TRUE,
    replace = FALSE,
    orderside = 'long',
    ordertype = 'stoptrailing', # 'stoplimit', # 'stoptrailing'
    tmult = TRUE,
    threshold = quote(.stopLevel),
    # TxnFees = .txnfees,
    orderqty = 'all',
    orderset = 'ocolong'
  ),
  type = 'chain',
  parent = 'EnterLONG',
  label = 'StopLossLONG',
  enabled = .enabledStopLoss
)


# Now that we have our strategy specified, let us run it. 
# Get begin time
t1 <- Sys.time()
out <- applyStrategy(strategy = strategy.st, portfolios = portfolio.st)
# Record end time 
t2 <- Sys.time() 
print(t2 - t1)
# The applyStrategy() function applies all of the indicators, signals, and 
# rules to the strategy. This creates a stream of orders for the specified 
# instruments. If everything works correctly, we will see a stream of orders. 

# getOrderBook(portfolio.st)
# View(getOrderBook(portfolio.st)[[portfolio.st]])
# View(mktdata["2016::"])
# View(getPortfolio(portfolio.st)$summary)
# tail(getPortfolio(portfolio.st))

###############################
# Evaluating the performance.
###############################
# The next step is to analyze the effectiveness of the implementation:
updatePortf(portfolio.st)
dateRange <- time(getPortfolio(portfolio.st)$summary)[-1] 
updateAcct(portfolio.st, dateRange) 
updateEndEq(account.st)

# Chart Positions
chart.Posn(portfolio.st, Symbol = symbols, TA = "add_EMA(200, col = 'red')")

library(dplyr)
library(knitr)
# Trade Statistics
tStats <- tradeStats(portfolio.st)
View(t(tStats))


